@@ -1,8 +1,14 @@
 import click
 import os
+import tqdm
 
-from aim.cli.runs.utils import list_repo_runs, match_runs, make_zip_archive, upload_repo_runs
+from multiprocessing.pool import ThreadPool
+from psutil import cpu_count
+
+from aim.cli.runs.utils import list_repo_runs, match_runs, make_zip_archive, upload_repo_runs, optimize_container
 from aim.sdk.repo import Repo
+from aim.sdk.lock_manager import LockManager
+from aim.sdk.index_manager import RepoIndexManager
 
 
 @click.group()
@@ -39,7 +45,8 @@ def list_runs(ctx):
 @runs.command(name='rm')
 @click.argument('hashes', nargs=-1, type=str)
 @click.pass_context
-def remove_runs(ctx, hashes):
+@click.option('-y', '--yes', is_flag=True, help='Automatically confirm prompt')
+def remove_runs(ctx, hashes, yes):
     """Remove Run data for given run hashes."""
     if len(hashes) == 0:
         click.echo('Please specify at least one Run to delete.')
@@ -48,8 +55,11 @@ def remove_runs(ctx, hashes):
     repo = Repo.from_path(repo_path)
 
     matched_hashes = match_runs(repo_path, hashes)
-    confirmed = click.confirm(f'This command will permanently delete {len(matched_hashes)} runs from aim repo '
-                              f'located at \'{repo_path}\'. Do you want to proceed?')
+    if yes:
+        confirmed = True
+    else:
+        confirmed = click.confirm(f'This command will permanently delete {len(matched_hashes)} runs from aim repo '
+                                  f'located at \'{repo_path}\'. Do you want to proceed?')
     if not confirmed:
         return
 
@@ -136,3 +146,47 @@ def upload_runs(ctx, bucket):
         click.echo(f'Successfully uploaded runs in {uploaded_zip_file_name}.')
     else:
         click.echo(f'The storage backup failed because of the following error: {uploaded_zip_file_name}.')
+
+
+@runs.command(name='close')
+@click.argument('hashes', nargs=-1, type=str)
+@click.pass_context
+@click.option('-y', '--yes', is_flag=True, help='Automatically confirm prompt')
+def close_runs(ctx, hashes, yes):
+    """Close failed/stalled Runs."""
+    repo_path = ctx.obj['repo']
+    repo = Repo.from_path(repo_path)
+
+    if len(hashes) == 0:
+        click.echo('Please specify at least one Run to close.')
+        exit(1)
+
+    click.secho(f'This command will forcefully close {len(hashes)} Runs from Aim Repo \'{repo_path}\'. '
+                f'Please make sure Runs are not active. Data corruption may occur otherwise.')
+    if yes:
+        confirmed = True
+    else:
+        confirmed = click.confirm('Do you want to proceed?')
+    if not confirmed:
+        return
+
+    lock_manager = LockManager(repo.path)
+    index_manager = RepoIndexManager.get_index_manager(repo)
+
+    def close_run(run_hash):
+        if lock_manager.release_locks(run_hash, force=True):
+            # Run rocksdb optimizations if container locks are removed
+            meta_db_path = os.path.join(repo.path, 'meta', 'chunks', run_hash)
+            seqs_db_path = os.path.join(repo.path, 'seqs', 'chunks', run_hash)
+            optimize_container(meta_db_path, extra_options={'compaction': True})
+            optimize_container(seqs_db_path, extra_options={})
+        if index_manager.run_needs_indexing(run_hash):
+            index_manager.index(run_hash)
+
+    pool = ThreadPool(cpu_count(logical=False))
+
+    for _ in tqdm.tqdm(
+            pool.imap_unordered(close_run, hashes),
+            desc='Closing runs',
+            total=len(hashes)):
+        pass
