@@ -5,7 +5,9 @@ from aim.cli.utils import set_log_level
 from aim.sdk.repo import Repo, RepoStatus
 from aim.sdk.utils import clean_repo_path
 from aim.ext.transport.config import AIM_SERVER_DEFAULT_HOST, AIM_SERVER_DEFAULT_PORT, AIM_SERVER_MOUNTED_REPO_PATH
-from aim.ext.transport.server import run_server
+from aim.ext.transport.server import run_router
+
+from aim.utils.tracking import analytics
 
 
 @click.command()
@@ -25,9 +27,10 @@ from aim.ext.transport.server import run_server
                                                                 dir_okay=False,
                                                                 readable=True))
 @click.option('--log-level', required=False, default='', type=str)
+@click.option('-y', '--yes', is_flag=True, help='Automatically confirm prompt')
 def server(host, port, workers,
            repo, ssl_keyfile, ssl_certfile,
-           log_level):
+           log_level, yes):
     # TODO [MV, AT] remove code duplication with aim up cmd implementation
     if log_level:
         set_log_level(log_level)
@@ -35,7 +38,10 @@ def server(host, port, workers,
     repo_path = clean_repo_path(repo) or Repo.default_repo_path()
     repo_status = Repo.check_repo_status(repo_path)
     if repo_status == RepoStatus.MISSING:
-        init_repo = click.confirm(f'\'{repo_path}\' is not a valid Aim repository. Do you want to initialize it?')
+        if yes:
+            init_repo = True
+        else:
+            init_repo = click.confirm(f'\'{repo_path}\' is not a valid Aim repository. Do you want to initialize it?')
 
         if not init_repo:
             click.echo('To initialize repo please run the following command:')
@@ -43,7 +49,10 @@ def server(host, port, workers,
             return
         repo_inst = Repo.from_path(repo_path, init=True)
     elif repo_status == RepoStatus.UPDATE_REQUIRED:
-        upgrade_repo = click.confirm(f'\'{repo_path}\' requires upgrade. Do you want to run upgrade automatically?')
+        if yes:
+            upgrade_repo = True
+        else:
+            upgrade_repo = click.confirm(f'\'{repo_path}\' requires upgrade. Do you want to run upgrade automatically?')
         if upgrade_repo:
             from aim.cli.upgrade.utils import convert_2to3
             repo_inst = convert_2to3(repo_path, drop_existing=False, skip_failed_runs=False, skip_checks=False)
@@ -61,9 +70,10 @@ def server(host, port, workers,
                     fg='yellow'))
     click.echo('Server is mounted on {}:{}'.format(host, port), err=True)
     click.echo('Press Ctrl+C to exit')
+    analytics.track_event(event_name='[Aim Remote Tracking] Start server')
 
     try:
-        run_server(host, port, workers, ssl_keyfile, ssl_certfile)
+        run_router(host, port, workers, ssl_keyfile, ssl_certfile)
     except Exception:
         click.echo('Failed to run Aim Tracking Server. '
                    'Please see the logs for details.')
