@@ -3,7 +3,7 @@ import struct
 import json
 from typing import Iterator, Tuple, Union
 
-import aim.ext.transport.remote_tracking_pb2 as rpc_messages
+import aim.ext.transport.proto.remote_tracking_pb2 as rpc_messages
 from aim.storage.object import CustomObject
 from aim.storage.types import BLOB
 
@@ -84,6 +84,11 @@ def build_exception(exception: Exception):
     )
 
 
+class UnauthorizedRequestError(RuntimeError):
+    def __init__(self, handler, *args, **kwargs):
+        self.handler = handler
+
+
 @CustomObject.alias('aim.resource')
 class ResourceObject(CustomObject):
     AIM_NAME = 'aim.resource'
