@@ -2,13 +2,12 @@ import os
 from typing import Any, Dict, Optional, Union
 from argparse import Namespace
 
+import packaging.version
+
 try:
     import pytorch_lightning as pl
 
-    def versiontuple(v):
-        return tuple(map(int, (v.split("."))))
-
-    if versiontuple(pl.__version__) < (1, 7):
+    if packaging.version.parse(pl.__version__) < packaging.version.parse("1.7"):
         from pytorch_lightning.loggers.base import (
             LightningLoggerBase as Logger,
             rank_zero_experiment,
@@ -41,11 +40,15 @@ class AimLogger(Logger):
                  test_metric_prefix: Optional[str] = 'test_',
                  system_tracking_interval: Optional[int]
                  = DEFAULT_SYSTEM_TRACKING_INT,
-                 log_system_params: bool = True,
+                 log_system_params: Optional[bool] = True,
+                 capture_terminal_logs: Optional[bool] = True,
+                 run_name: Optional[str] = None,
+                 run_hash: Optional[str] = None,
                  ):
         super().__init__()
 
         self._experiment_name = experiment
+        self._run_name = run_name
         self._repo_path = repo
 
         self._train_metric_prefix = train_metric_prefix
@@ -53,9 +56,10 @@ class AimLogger(Logger):
         self._test_metric_prefix = test_metric_prefix
         self._system_tracking_interval = system_tracking_interval
         self._log_system_params = log_system_params
+        self._capture_terminal_logs = capture_terminal_logs
 
         self._run = None
-        self._run_hash = None
+        self._run_hash = run_hash
 
     @staticmethod
     def _convert_params(params: Union[Dict[str, Any], Namespace]) -> Dict[str, Any]:
@@ -77,13 +81,17 @@ class AimLogger(Logger):
                     self._run_hash,
                     repo=self._repo_path,
                     system_tracking_interval=self._system_tracking_interval,
+                    capture_terminal_logs=self._capture_terminal_logs
                 )
+                if self._run_name is not None:
+                    self._run.name = self._run_name
             else:
                 self._run = Run(
                     repo=self._repo_path,
                     experiment=self._experiment_name,
                     system_tracking_interval=self._system_tracking_interval,
                     log_system_params=self._log_system_params,
+                    capture_terminal_logs=self._capture_terminal_logs,
                 )
                 self._run_hash = self._run.hash
         return self._run
@@ -111,7 +119,14 @@ class AimLogger(Logger):
         assert rank_zero_only.rank == 0, \
             'experiment tried to log from global_rank != 0'
 
-        for k, v in metrics.items():
+        metric_items: Dict[str: Any] = {k: v for k, v in metrics.items()}
+
+        if 'epoch' in metric_items:
+            epoch: int = metric_items.pop('epoch')
+        else:
+            epoch = None
+
+        for k, v in metric_items.items():
             name = k
             context = {}
             if self._train_metric_prefix \
@@ -126,7 +141,7 @@ class AimLogger(Logger):
                     and name.startswith(self._val_metric_prefix):
                 name = name[len(self._val_metric_prefix):]
                 context['subset'] = 'val'
-            self.experiment.track(v, name=name, step=step, context=context)
+            self.experiment.track(v, name=name, step=step, epoch=epoch, context=context)
 
     @rank_zero_only
     def finalize(self, status: str = '') -> None:
