@@ -16,20 +16,24 @@ except ImportError:
 class AimCallback(TrackerKerasCallbackMetricsEpochEndMixin, Callback):
     def __init__(self, repo: Optional[str] = None,
                  experiment: Optional[str] = None,
-                 system_tracking_interval: int = DEFAULT_SYSTEM_TRACKING_INT,
-                 log_system_params: bool = True,):
+                 system_tracking_interval: Optional[int] = DEFAULT_SYSTEM_TRACKING_INT,
+                 log_system_params: Optional[bool] = True,
+                 capture_terminal_logs: Optional[bool] = True,):
         super(Callback, self).__init__()
 
         self._system_tracking_interval = system_tracking_interval
         self._log_system_params = log_system_params
+        self._capture_terminal_logs = capture_terminal_logs
 
         if repo is None and experiment is None:
             self._run = Run(system_tracking_interval=self._system_tracking_interval,
-                            log_system_params=self._log_system_params,)
+                            log_system_params=self._log_system_params,
+                            capture_terminal_logs=self._capture_terminal_logs,)
         else:
             self._run = Run(repo=repo, experiment=experiment,
                             system_tracking_interval=self._system_tracking_interval,
-                            log_system_params=self._log_system_params,)
+                            log_system_params=self._log_system_params,
+                            capture_terminal_logs=self._capture_terminal_logs,)
 
         self._run_hash = self._run.hash
         self._repo_path = repo
@@ -39,7 +43,8 @@ class AimCallback(TrackerKerasCallbackMetricsEpochEndMixin, Callback):
         if not self._run:
             self._run = Run(self._run_hash,
                             repo=self._repo_path,
-                            system_tracking_interval=self._system_tracking_interval,)
+                            system_tracking_interval=self._system_tracking_interval,
+                            capture_terminal_logs=self._capture_terimanl_logs)
         return self._run
 
     @classmethod
