@@ -83,4 +83,6 @@ class Figure(CustomObject):
 
 
 def matplotlib_get_offset_position(self):
-    return self._offset_position
+    # self._offset_position is depricated
+    # using 'screen' as the default value
+    return 'screen'
