@@ -1,4 +1,13 @@
 from aim.storage.object import CustomObject
+import logging
+from pathlib import Path
+import yaml
+
+try:
+    from dvc.repo import Repo
+    import dvc.api as api
+except ImportError:
+    raise ImportError("module dvc could not be imported")
 
 
 @CustomObject.alias('dvc.metadata')
@@ -17,8 +26,11 @@ class DvcData(CustomObject):
         """
 
         super().__init__()
+
         self.storage['dataset'] = {
             'source': 'dvc',
+            'version': self._get_dvc_lock(url),
+            'params': self._get_dvc_params(url),
             'tracked_files': self._get_dvc_tracked_files(
                 **dict(
                     url=url,
@@ -27,16 +39,28 @@ class DvcData(CustomObject):
                     recursive=recursive,
                     dvc_only=dvc_only
                 )
-            )
+            ),
         }
 
     def _get_dvc_tracked_files(self, **ls_kwargs):
-        try:
-            from dvc.repo import Repo
-        except ImportError:
-            raise RuntimeError(
-                'Could not import module dvc.'
-            )
-
         entries = Repo.ls(**ls_kwargs)
         return [entry['path'] for entry in entries]
+
+    def _get_dvc_params(self, url):
+        try:
+            params = api.params_show(repo=url)
+            return params
+        except Exception:
+            logging.warning("Failed to log params")
+
+    def _get_dvc_lock(self, url):
+        try:
+            with open(Path(url).joinpath('dvc.lock'), 'r') as f:
+                try:
+                    content = yaml.safe_load(f)
+                    return content
+                except yaml.YAMLError as exc:
+                    logging.warning(exc)
+                content = f.readlines()
+        except FileNotFoundError:
+            logging.warning(f"Failed to find dvc.lock in the repo {url}")
