@@ -23,12 +23,14 @@ from aim.sdk.sequence import Sequence
 from aim.sdk.types import QueryReportMode
 from aim.sdk.data_version import DATA_VERSION
 from aim.sdk.remote_repo_proxy import RemoteRepoProxy
+from aim.sdk.lock_manager import LockManager, RunLock
 
-from aim.storage.locking import AutoFileLock
+from aim.storage.locking import SoftFileLock
 from aim.storage.container import Container
-from aim.storage.rockscontainer import RocksContainer
+from aim.storage.rockscontainer import RocksContainer, LockableRocksContainer
 from aim.storage.union import RocksUnionContainer
 from aim.storage.treeviewproxy import ProxyTree
+from aim.storage.lock_proxy import ProxyLock
 
 from aim.storage.structured.db import DB
 from aim.storage.structured.proxy import StructuredRunProxy
@@ -74,6 +76,7 @@ class RepoAutoClean(AutoClean):
         """Close the `Repo` and unmount the remote repository."""
         if self._client:
             self._client._heartbeat_sender.stop()
+            self._client.disconnect()
         if self._mount_root:
             logger.debug(f'Unmounting remote repository at {self._mount_root}')
             unmount_remote_repo(self.root_path, self._mount_root)
@@ -88,7 +91,7 @@ class Repo:
     Provides API for querying Runs/Metrics based on a given expression.
 
     Args:
-        path (str): Path to Aim repository.
+        path (:obj:`str`): Path to Aim repository.
         read_only (:obj:`bool`, optional): Flag for opening Repo in readonly mode. False by default.
         init (:obj:`bool`, optional): Flag used to initialize new Repo. False by default.
             Recommended to use ``aim init`` command instead.
@@ -105,19 +108,21 @@ class Repo:
         self.read_only = read_only
         self._mount_root = None
         self._client: Client = None
+        self._lock_manager: LockManager = None
         if path.startswith('ssh://'):
             self._mount_root, self.root_path = mount_remote_repo(path)
         elif self.is_remote_path(path):
             remote_path = path.replace('aim://', '')
             self._client = Client(remote_path)
+            self._remote_repo_proxy = RemoteRepoProxy(self._client)
             self.root_path = remote_path
-            self._check_remote_version_compatibility()
         else:
             self.root_path = path
         self.path = os.path.join(self.root_path, get_aim_repo_name())
 
         if init:
             os.makedirs(self.path, exist_ok=True)
+            os.makedirs(os.path.join(self.path, 'locks'), exist_ok=True)
         if not self.is_remote_repo and not os.path.exists(self.path):
             if self._mount_root:
                 unmount_remote_repo(self.root_path, self._mount_root)
@@ -132,13 +137,14 @@ class Repo:
         self.structured_db = None
 
         if not self.is_remote_repo:
-            self._lock_path = os.path.join(self.path, '.repo_lock')
-            self._lock = AutoFileLock(self._lock_path, timeout=10)
+            self._lock_manager = LockManager(self.path)
+            self._sdb_lock_path = os.path.join(self.path, 'locks', 'structured_db_lock')
+            self._sdb_lock = SoftFileLock(self._sdb_lock_path, timeout=5 * 60)  # timeout after 5 minutes
 
-            with self.lock():
-                status = self.check_repo_status(self.root_path)
-                self.structured_db = DB.from_path(self.path)
-                if init or status == RepoStatus.PATCH_REQUIRED:
+            status = self.check_repo_status(self.root_path)
+            self.structured_db = DB.from_path(self.path)
+            if init or status == RepoStatus.PATCH_REQUIRED:
+                with self._sdb_lock:
                     self.structured_db.run_upgrades()
                     with open(os.path.join(self.path, 'VERSION'), 'w') as version_fh:
                         version_fh.write('.'.join(map(str, DATA_VERSION)) + '\n')
@@ -158,16 +164,6 @@ class Repo:
     def __eq__(self, o: 'Repo') -> bool:
         return self.path == o.path
 
-    @contextmanager
-    def lock(self):
-        assert not self.is_remote_repo
-
-        self._lock.acquire()
-        try:
-            yield self._lock
-        finally:
-            self._lock.release()
-
     @classmethod
     def default_repo_path(cls) -> str:
         repo_path, found = search_aim_repo(os.path.curdir)
@@ -195,7 +191,7 @@ class Repo:
         """Named constructor for Repo for given path.
 
         Arguments:
-            path (str): Path to Aim repository.
+            path (:obj:`str`): Path to Aim repository.
             read_only (:obj:`bool`, optional): Flag for opening Repo in readonly mode. False by default.
             init (:obj:`bool`, optional): Flag used to initialize new Repo. False by default.
                 Recommended to use ``aim init`` command instead.
@@ -215,7 +211,7 @@ class Repo:
         """Check Aim repository existence.
 
         Args:
-            path (str): Path to Aim repository.
+            path (:obj:`str`): Path to Aim repository.
         Returns:
             True if repository exists, False otherwise.
         """
@@ -228,7 +224,7 @@ class Repo:
         """Remove Aim repository.
 
         Args:
-            path (str): Path to Aim repository.
+            path (:obj:`str`): Path to Aim repository.
         """
         path = clean_repo_path(path)
         repo = cls._pool.get(path)
@@ -252,43 +248,6 @@ class Repo:
                 return RepoStatus.PATCH_REQUIRED
         return RepoStatus.UPDATED
 
-    def _check_remote_version_compatibility(self):
-        assert self.is_remote_repo
-        from aim.__version__ import __version__ as client_version
-        import grpc
-
-        error_message_template = 'The Aim Remote tracking server version ({}) '\
-                                 'is not compatible with the Aim client version ({}).'\
-                                 'Please upgrade either the Aim Client or the Aim Remote.'
-
-        warning_message_template = 'The Aim Remote tracking server version ({}) ' \
-                                   'and the Aim client version ({}) do not match.' \
-                                   'Consider upgrading either the client or remote tracking server.'
-
-        try:
-            remote_version = self._client.get_version()
-        except grpc.RpcError as e:
-            if e.code() == grpc.StatusCode.UNIMPLEMENTED:
-                remote_version = '<3.12.0'
-            else:
-                raise
-
-        # server doesn't yet have the `get_version()` method implemented
-        if remote_version == '<3.12.0':
-            RuntimeError(error_message_template.format(remote_version, client_version))
-
-        # compare versions
-        if client_version == remote_version:
-            return
-
-        # if the server has a newer version always force to upgrade the client
-        if client_version < remote_version:
-            raise RuntimeError(error_message_template.format(remote_version, client_version))
-
-        # for other mismatching versions throw a warning for now
-        logger.warning(warning_message_template.format(remote_version, client_version))
-        # further incompatibility list will be added manually
-
     @classmethod
     def get_version(cls, path: str):
         path = clean_repo_path(path)
@@ -337,7 +296,7 @@ class Repo:
         container = self.container_pool.get(container_config)
         if container is None:
             path = os.path.join(self.path, name)
-            container = RocksContainer(path, read_only=False, timeout=timeout)
+            container = LockableRocksContainer(path, read_only=False, timeout=timeout)
             self.container_pool[container_config] = container
 
         return container
@@ -348,12 +307,13 @@ class Repo:
         sub: str = None,
         *,
         read_only: bool,
-        from_union: bool = False  # TODO maybe = True by default
+        from_union: bool = False,  # TODO maybe = True by default
+        no_cache: bool = False,
     ):
         if not self.is_remote_repo:
-            return self.request(name, sub, read_only=read_only, from_union=from_union).tree()
+            return self.request(name, sub, read_only=read_only, from_union=from_union, no_cache=no_cache).tree()
         else:
-            return ProxyTree(self._client, name, sub, read_only, from_union)
+            return ProxyTree(self._client, name, sub, read_only=read_only, from_union=from_union, no_cache=no_cache)
 
     def request(
             self,
@@ -361,12 +321,13 @@ class Repo:
             sub: str = None,
             *,
             read_only: bool,
-            from_union: bool = False  # TODO maybe = True by default
+            from_union: bool = False,  # TODO maybe = True by default
+            no_cache: bool = False,
     ):
 
         container_config = ContainerConfig(name, sub, read_only)
         container_view = self.container_view_pool.get(container_config)
-        if container_view is None:
+        if container_view is None or no_cache:
             if read_only:
                 if from_union:
                     path = name
@@ -380,7 +341,8 @@ class Repo:
                 container = self._get_container(path, read_only=False, from_union=False)
 
             container_view = container
-            self.container_view_pool[container_config] = container_view
+            if not no_cache:
+                self.container_view_pool[container_config] = container_view
 
         return container_view
 
@@ -391,21 +353,27 @@ class Repo:
         assert self.structured_db
         _props = None
 
-        with self.lock():
-            if self.run_props_cache_hint:
-                _props = self.structured_db.caches[self.run_props_cache_hint][hash_]
+        if self.run_props_cache_hint:
+            _props = self.structured_db.caches[self.run_props_cache_hint][hash_]
+        if not _props:
+            _props = self.structured_db.find_run(hash_)
             if not _props:
-                _props = self.structured_db.find_run(hash_)
-                if not _props:
-                    if read_only:
-                        raise RepoIntegrityError(f'Missing props for Run {hash_}')
-                    else:
+                if read_only:
+                    raise RepoIntegrityError(f'Missing props for Run {hash_}')
+                else:
+                    with self._sdb_lock:
                         _props = self.structured_db.create_run(hash_)
-                if self.run_props_cache_hint:
-                    self.structured_db.caches[self.run_props_cache_hint][hash_] = _props
+            if self.run_props_cache_hint:
+                self.structured_db.caches[self.run_props_cache_hint][hash_] = _props
 
         return _props
 
+    def request_run_lock(self, hash_: str, timeout: int = 10) -> 'RunLock':
+        if self.is_remote_repo:
+            return ProxyLock(self._client, hash_)
+        assert self._lock_manager
+        return self._lock_manager.get_run_lock(hash_, timeout=timeout)
+
     def iter_runs(self) -> Iterator['Run']:
         """Iterate over Repo runs.
 
@@ -436,8 +404,7 @@ class Repo:
     @ttl_cache(ttl=0.5)
     def _all_run_hashes(self) -> Set[str]:
         if self.is_remote_repo:
-            remote_repo = RemoteRepoProxy(self._client)
-            return set(remote_repo.list_all_runs())
+            return set(self._remote_repo_proxy.list_all_runs())
         else:
             chunks_dir = os.path.join(self.path, 'meta', 'chunks')
             if os.path.exists(chunks_dir):
@@ -450,8 +417,7 @@ class Repo:
 
     def _active_run_hashes(self) -> Set[str]:
         if self.is_remote_repo:
-            remote_repo = RemoteRepoProxy(self._client)
-            return set(remote_repo.list_active_runs())
+            return set(self._remote_repo_proxy.list_active_runs())
         else:
             chunks_dir = os.path.join(self.path, 'meta', 'progress')
             if os.path.exists(chunks_dir):
@@ -478,7 +444,7 @@ class Repo:
         """Get run if exists.
 
         Args:
-            run_hash (str): Run hash.
+            run_hash (:obj:`str`): Run hash.
         Returns:
             :obj:`Run` object if hash is found in repository. `None` otherwise.
         """
@@ -607,7 +573,7 @@ class Repo:
         """Get metrics satisfying query expression.
 
         Args:
-             query (str): query expression.
+             query (:obj:`str`): query expression.
              report_mode(:obj:`QueryReportMode`, optional): indicates report mode
                 (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
@@ -623,7 +589,7 @@ class Repo:
         """Get image collections satisfying query expression.
 
         Args:
-             query (str): query expression.
+             query (:obj:`str`): query expression.
              report_mode(:obj:`QueryReportMode`, optional): indicates report mode
                 (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
@@ -639,7 +605,7 @@ class Repo:
         """Get audio collections satisfying query expression.
 
         Args:
-             query (str): query expression.
+             query (:obj:`str`): query expression.
              report_mode(:obj:`QueryReportMode`, optional): indicates report mode
                 (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
@@ -655,7 +621,7 @@ class Repo:
         """Get Figures collections satisfying query expression.
 
         Args:
-             query (str): query expression.
+             query (:obj:`str`): query expression.
              report_mode(:obj:`QueryReportMode`, optional): indicates report mode
                 (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
@@ -671,7 +637,7 @@ class Repo:
         """Get distribution collections satisfying query expression.
 
         Args:
-             query (str): query expression.
+             query (:obj:`str`): query expression.
              report_mode(:obj:`QueryReportMode`, optional): indicates report mode
                 (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
@@ -687,7 +653,7 @@ class Repo:
         """Get text collections satisfying query expression.
 
         Args:
-             query (str): query expression.
+             query (:obj:`str`): query expression.
              report_mode(:obj:`QueryReportMode`, optional): indicates report mode
                 (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
@@ -746,7 +712,7 @@ class Repo:
 
         Args:
             sequence_types (:obj:`tuple[str]`, optional): Sequence types to get tracked sequence names/contexts for.
-            Defaults to 'metric'.
+                Defaults to 'metric'.
 
         Returns:
             :obj:`dict`: Tree of sequences and their contexts groupped by sequence type.
@@ -807,11 +773,9 @@ class Repo:
         self.run_props_cache_hint = cache_name
 
     def _delete_run(self, run_hash):
-        # try to acquire a lock on a run container to check if it is still in progress or not
-        # in progress runs can't be deleted
-        lock_path = os.path.join(self.path, 'meta', 'locks', run_hash)
-        lock = AutoFileLock(lock_path, timeout=0)
-        lock.acquire()
+        # check run lock info. in progress runs can't be deleted
+        if self._lock_manager.get_run_lock_info(run_hash).locked:
+            raise RuntimeError(f'Cannot delete Run \'{run_hash}\'. Run is locked.')
 
         with self.structured_db:  # rollback db entity delete if subsequent actions fail.
             # remove database entry
@@ -836,11 +800,10 @@ class Repo:
                     shutil.rmtree(seqs_path, ignore_errors=True)
 
     def _copy_run(self, run_hash, dest_repo):
-        # try to acquire a lock on a run container to check if it is still in progress or not
-        # in progress runs can't be copied
-        lock_path = os.path.join(self.path, 'meta', 'locks', run_hash)
-        lock = AutoFileLock(lock_path, timeout=0)
-        lock.acquire()
+        # check run lock info. in progress runs can't be copied
+        if self._lock_manager.get_run_lock_info(run_hash).locked:
+            raise RuntimeError(f'Cannot copy Run \'{run_hash}\'. Run is locked.')
+
         with dest_repo.structured_db:  # rollback destination db entity if subsequent actions fail.
             # copy run structured data
             source_structured_run = self.structured_db.find_run(run_hash)
