@@ -1,5 +1,6 @@
 import logging
 from typing import Optional, Union, TYPE_CHECKING
+import pathlib
 
 if TYPE_CHECKING:
     from aim.sdk.repo import Repo
@@ -7,11 +8,13 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-def get_repo(repo: Optional[Union[str, 'Repo']]) -> 'Repo':
+def get_repo(repo: Optional[Union[str, 'Repo', pathlib.Path]]) -> 'Repo':
     from aim.sdk.repo import Repo, RepoStatus
 
     if repo is None:
         repo = Repo.default_repo_path()
+    if isinstance(repo, pathlib.Path):
+        repo = str(repo)
     if isinstance(repo, str):
         repo_status = Repo.check_repo_status(repo)
         if repo_status == RepoStatus.UPDATE_REQUIRED:
