@@ -2,6 +2,7 @@ import pytz
 
 from typing import Collection, Union, List, Optional
 from sqlalchemy.orm import joinedload
+from sqlalchemy.exc import IntegrityError
 
 from aim.storage.types import SafeNone
 from aim.storage.structured.entities import (
@@ -9,12 +10,14 @@ from aim.storage.structured.entities import (
     Experiment as IExperiment,
     Tag as ITag,
     Note as INote,
+    RunInfo as IRunInfo,
     RunCollection,
     TagCollection,
     NoteCollection
 )
 from aim.storage.structured.sql_engine.models import (
     Run as RunModel,
+    RunInfo as RunInfoModel,
     Experiment as ExperimentModel,
     Tag as TagModel,
     Note as NoteModel,
@@ -47,6 +50,7 @@ class ModelMappedRun(IRun, metaclass=ModelMappedClassMeta):
         Property('experiment', autogenerate=False),
         Property('tags', autogenerate=False),
         Property('notes', autogenerate=False),
+        Property('info', autogenerate=False),
     ]
 
     def __init__(self, model: RunModel, session):
@@ -135,19 +139,42 @@ class ModelMappedRun(IRun, metaclass=ModelMappedClassMeta):
     def experiment(self) -> Union[str, SafeNone]:
         return self.experiment_obj.name if self.experiment_obj else SafeNone()
 
+    @property
+    def info(self) -> Optional[IRunInfo]:
+        if self._model:
+            if self._model.info:
+                return ModelMappedRunInfo(self._model.info, self._session)
+            else:
+                info = RunInfoModel()
+
+                self._model.info = info
+                self._session.add(info)
+                self._session.add(self._model)
+                self._session.flush()
+
+                return ModelMappedRunInfo(self._model.info, self._session)
+
     @experiment.setter
     def experiment(self, value: str):
+        def unsafe_set_exp():
+            if value is None:
+                exp = None
+            else:
+                exp = session.query(ExperimentModel).filter(ExperimentModel.name == value).first()
+                if not exp:
+                    exp = ExperimentModel(value)
+                    session.add(exp)
+            self._model.experiment = exp
+            session.add(self._model)
+
         session = self._session
-        if value is None:
-            exp = None
-        else:
-            exp = session.query(ExperimentModel).filter(ExperimentModel.name == value).first()
-            if not exp:
-                exp = ExperimentModel(value)
-                session.add(exp)
-        self._model.experiment = exp
-        session.add(self._model)
-        session.flush()
+        unsafe_set_exp()
+        try:
+            session.flush()
+        except IntegrityError:
+            session.rollback()
+            unsafe_set_exp()
+            session.flush()
 
     @property
     def tags_obj(self) -> TagCollection:
@@ -266,10 +293,13 @@ class ModelMappedExperiment(IExperiment, metaclass=ModelMappedClassMeta):
     __model__ = ExperimentModel
     __mapped_properties__ = [
         Property('name'),
+        Property('description'),
         Property('uuid', with_setter=False),
         Property('archived', 'is_archived'),
         Property('created_at', with_setter=False),
+        Property('creation_time', 'created_at', get_modifier=timestamp_or_none, with_setter=False),
         Property('updated_at', with_setter=False),
+        Property('notes', autogenerate=False)
     ]
 
     def __init__(self, model_inst: ExperimentModel, session):
@@ -340,6 +370,79 @@ class ModelMappedExperiment(IExperiment, metaclass=ModelMappedClassMeta):
         ]).filter(ExperimentModel.name.like(term))
         return ModelMappedExperimentCollection(session, query=q)
 
+    @property
+    def notes(self) -> List[dict]:
+        session = self._session
+
+        qs = session.query(NoteModel).filter(
+            NoteModel.experiment_id == self._model.id,
+        ).order_by(NoteModel.updated_at.desc())
+
+        return [{
+            "id": note.id,
+            "content": note.content,
+            "created_at": note.created_at,
+            "updated_at": note.updated_at
+        } for note in qs]
+
+    def find_note(self, _id: int):
+        session = self._session
+
+        qs = session.query(NoteModel).filter(
+            NoteModel.experiment_id == self._model.id,
+            NoteModel.id == _id
+        )
+        return qs.first()
+
+    def add_note(self, content: str):
+        session = self._session
+
+        note = NoteModel(content)
+        session.add(note)
+        self._model.notes.append(note)
+
+        audit_log = NoteAuditLogModel(action="Created", before=None, after=content)
+        session.add(audit_log)
+        note.audit_logs.append(audit_log)
+
+        session.add(self._model)
+        session.flush()
+
+        return note
+
+    def update_note(self, _id: int, content):
+        session = self._session
+
+        note = self.find_note(_id=_id)
+
+        before = note.content
+        note.content = content
+
+        audit_log = NoteAuditLogModel(action="Updated", before=before, after=content)
+        session.add(audit_log)
+        note.audit_logs.append(audit_log)
+
+        session.add(note)
+        session.flush()
+
+        return note
+
+    def remove_note(self, _id: int):
+        session = self._session
+
+        audit_log = NoteAuditLogModel(action="Deleted", before=None, after=None)
+        audit_log.note_id = _id
+        session.add(audit_log)
+
+        session.query(NoteModel).filter(
+            NoteModel.experiment_id == self._model.id,
+            NoteModel.id == _id,
+        ).delete()
+        session.flush()
+
+    def refresh_model(self):
+        self._session.refresh(self._model)
+
 
 class ModelMappedTag(ITag, metaclass=ModelMappedClassMeta):
     __model__ = TagModel
@@ -493,6 +596,23 @@ class ModelMappedNote(INote, metaclass=ModelMappedClassMeta):
         return False
 
 
+class ModelMappedRunInfo(IRunInfo, metaclass=ModelMappedClassMeta):
+    __model__ = RunInfoModel
+    __mapped_properties__ = [
+        Property('created_at', with_setter=False),
+        Property('updated_at', with_setter=False),
+        Property('last_notification_index'),
+    ]
+
+    def __init__(self, model_inst: RunInfoModel, session):
+        self._model = model_inst
+        self._id = model_inst.id
+        self._session = session
+
+    def __repr__(self) -> str:
+        return f'<ModelMappedRunInfo id={self._id}>'
+
+
 ModelMappedRunCollection = ModelMappedCollection[ModelMappedRun]
 ModelMappedExperimentCollection = ModelMappedCollection[ModelMappedExperiment]
 ModelMappedTagCollection = ModelMappedCollection[ModelMappedTag]
