@@ -3,7 +3,11 @@ AIM_UI_DEFAULT_PORT = 43800
 AIM_UI_DEFAULT_HOST = '127.0.0.1'
 AIM_TF_LOGS_PATH_KEY = '__AIM_TF_LOGS_PATH__'
 AIM_ENV_MODE_KEY = '__AIM_ENV_MODE__'
+AIM_LOG_LEVEL_KEY = '__AIM_LOG_LEVEL__'
 AIM_UI_MOUNTED_REPO_PATH = '__AIM_UI_MOUNT_REPO_PATH__'
 AIM_UI_TELEMETRY_KEY = 'AIM_UI_TELEMETRY_ENABLED'
 AIM_UI_BASE_PATH = '__AIM_UI_BASE_PATH__'
 AIM_PROXY_URL = '__AIM_PROXY_URL__'
+AIM_PROFILER_KEY = '__AIM_PROFILER_ENABLED__'
+AIM_PROGRESS_REPORT_INTERVAL = 0.5
+AIM_PROJECT_SETTINGS_FILE = '.project_settings'
