@@ -18,7 +18,7 @@ class TestRunApi(PrefilledDataApiTestBase):
                                                                'report_progress': False})
         self.assertEqual(200, response.status_code)
 
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=512*1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=512*1024)))
         self.assertEqual(1, len(decoded_response))
         for _, run in decoded_response.items():
             self.assertEqual(4, len(run['traces']['metric']))
@@ -33,7 +33,7 @@ class TestRunApi(PrefilledDataApiTestBase):
                                                                'limit': 1, 'report_progress': False})
         self.assertEqual(200, response.status_code)
 
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=1024 * 1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=1024 * 1024)))
         self.assertEqual(1, len(decoded_response))
 
         offset = ''
@@ -48,7 +48,7 @@ class TestRunApi(PrefilledDataApiTestBase):
                                                                'report_progress': False})
         self.assertEqual(200, response.status_code)
 
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=1024 * 1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=1024 * 1024)))
         self.assertEqual(1, len(decoded_response))
         for run_hash, run in decoded_response.items():
             self.assertEqual('Run # 2', run['props']['name'])
@@ -61,7 +61,7 @@ class TestRunApi(PrefilledDataApiTestBase):
                                                                   'report_progress': False})
         self.assertEqual(200, response.status_code)
 
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=512*1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=512*1024)))
         for run in decoded_response.values():
             for trace in run['traces']:
                 self.assertEqual([0, 0, 50], trace['slice'])
@@ -89,7 +89,7 @@ class TestRunApi(PrefilledDataApiTestBase):
                                                                   'report_progress': False})
         self.assertEqual(200, response.status_code)
 
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=512*1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=512*1024)))
         for run in decoded_response.values():
             for trace in run['traces']:
                 self.assertEqual([0, 0, step_count], trace['slice'])
@@ -121,7 +121,7 @@ class TestRunApi(PrefilledDataApiTestBase):
         })
         self.assertEqual(200, response.status_code)
 
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=512*1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=512*1024)))
         self.assertEqual(2, len(decoded_response))
         self.assertListEqual(run_hashes, list(decoded_response.keys()))
         self.assertEqual([], decoded_response[run_hashes[1]])
@@ -148,7 +148,7 @@ class TestRunApi(PrefilledDataApiTestBase):
             }]
         })
         self.assertEqual(200, response.status_code)
-        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_content(chunk_size=1024 * 1024)))
+        decoded_response = decode_tree(decode_encoded_tree_stream(response.iter_bytes(chunk_size=1024 * 1024)))
         self.assertEqual(2, len(decoded_response))
         for run in decoded_response.values():
             self.assertEqual([], run)
